@@ -20,16 +20,10 @@ import java.util.Hashtable;
 
 import javax.annotation.Nonnull;
 import javax.servlet.ServletContext;
-import javax.servlet.http.HttpSessionAttributeListener;
-import javax.servlet.http.HttpSessionListener;
 
 import org.apache.felix.http.api.ExtHttpService;
 import org.apache.felix.http.base.internal.registry.HandlerRegistry;
-import org.apache.felix.http.base.internal.service.listener.HttpSessionAttributeListenerManager;
-import org.apache.felix.http.base.internal.service.listener.HttpSessionListenerManager;
-import org.apache.felix.http.base.internal.service.listener.ServletContextAttributeListenerManager;
-import org.apache.felix.http.base.internal.service.listener.ServletRequestAttributeListenerManager;
-import org.apache.felix.http.base.internal.service.listener.ServletRequestListenerManager;
+import org.apache.felix.http.base.internal.service.listener.AbstractListenerManager;
 import org.osgi.framework.Bundle;
 import org.osgi.framework.BundleContext;
 import org.osgi.framework.Constants;
@@ -73,18 +67,14 @@ public final class HttpServiceFactory
     private final BundleContext bundleContext;
     private final boolean sharedContextAttributes;
 
-    private final ServletContextAttributeListenerManager contextAttributeListenerManager;
-    private final ServletRequestListenerManager requestListenerManager;
-    private final ServletRequestAttributeListenerManager requestAttributeListenerManager;
-    private final HttpSessionListenerManager sessionListenerManager;
-    private final HttpSessionAttributeListenerManager sessionAttributeListenerManager;
-
     private final Hashtable<String, Object> httpServiceProps = new Hashtable<String, Object>();
     private volatile ServletContext context;
     private volatile ServiceRegistration<?> httpServiceReg;
 
     private final HandlerRegistry handlerRegistry;
     private volatile SharedHttpServiceImpl sharedHttpService;
+    private volatile AbstractListenerManager listenerManager;
+
 
     public HttpServiceFactory(final BundleContext bundleContext,
             final HandlerRegistry handlerRegistry)
@@ -93,11 +83,6 @@ public final class HttpServiceFactory
         this.handlerRegistry = handlerRegistry;
         this.sharedContextAttributes = getBoolean(FELIX_HTTP_SHARED_SERVLET_CONTEXT_ATTRIBUTES);
 
-        this.contextAttributeListenerManager = new ServletContextAttributeListenerManager(bundleContext);
-        this.requestListenerManager = new ServletRequestListenerManager(bundleContext);
-        this.requestAttributeListenerManager = new ServletRequestAttributeListenerManager(bundleContext);
-        this.sessionListenerManager = new HttpSessionListenerManager(bundleContext);
-        this.sessionAttributeListenerManager = new HttpSessionAttributeListenerManager(bundleContext);
     }
 
     public void start(final ServletContext context,
@@ -113,11 +98,10 @@ public final class HttpServiceFactory
         }
 
         this.context = context;
-        this.contextAttributeListenerManager.open();
-        this.requestListenerManager.open();
-        this.requestAttributeListenerManager.open();
-        this.sessionListenerManager.open();
-        this.sessionAttributeListenerManager.open();
+        this.listenerManager = new AbstractListenerManager(bundleContext,
+                handlerRegistry
+                    .getRegistry(HTTP_SERVICE_CONTEXT_SERVICE_ID).getEventListenerRegistry());
+        this.listenerManager.open();
 
         this.sharedHttpService = new SharedHttpServiceImpl(handlerRegistry);
 
@@ -139,11 +123,11 @@ public final class HttpServiceFactory
         this.context = null;
         this.sharedHttpService = null;
 
-        this.contextAttributeListenerManager.close();
-        this.requestListenerManager.close();
-        this.requestAttributeListenerManager.close();
-        this.sessionListenerManager.close();
-        this.sessionAttributeListenerManager.close();
+        if ( this.listenerManager != null )
+        {
+            this.listenerManager.close();
+            this.listenerManager = null;
+        }
 
         this.httpServiceProps.clear();
     }
@@ -155,22 +139,17 @@ public final class HttpServiceFactory
         if (active) {
             // Store everything that we want to pass to the PerBundleHttpServiceImpl in local vars to avoid
             // a race condition where the service might be stopped while this method is executing.
-            SharedHttpServiceImpl sharedHttpSvc = this.sharedHttpService;
-            ServletContext servletCtx = this.context;
-            ServletContextAttributeListenerManager servletCtxAttrListenerMgr = this.contextAttributeListenerManager;
-            boolean sharedCtxAttrs = this.sharedContextAttributes;
-            ServletRequestListenerManager reqListenerMgr = this.requestListenerManager;
-            ServletRequestAttributeListenerManager reqAttrListenerMgr = this.requestAttributeListenerManager;
+            final SharedHttpServiceImpl sharedHttpSvc = this.sharedHttpService;
+            final ServletContext servletCtx = this.context;
+
+            final boolean sharedCtxAttrs = this.sharedContextAttributes;
 
             if ( active ) {
                 // Only return the service if we're still active
                 return new PerBundleHttpServiceImpl(bundle,
                         sharedHttpSvc,
                         servletCtx,
-                        servletCtxAttrListenerMgr,
-                        sharedCtxAttrs,
-                        reqListenerMgr,
-                        reqAttrListenerMgr);
+                        sharedCtxAttrs);
             }
         }
         return null;
@@ -186,21 +165,6 @@ public final class HttpServiceFactory
         }
     }
 
-    public ServletContextAttributeListenerManager getContextAttributeListener()
-    {
-        return contextAttributeListenerManager;
-    }
-
-    public HttpSessionListener getSessionListener()
-    {
-        return sessionListenerManager;
-    }
-
-    public HttpSessionAttributeListener getSessionAttributeListener()
-    {
-        return sessionAttributeListenerManager;
-    }
-
     public long getHttpServiceServiceId()
     {
         return (Long) this.httpServiceReg.getReference().getProperty(Constants.SERVICE_ID);
